@@ -783,12 +783,6 @@ int main (int argc, char **argv)
         << "enabled (" << FORTRAN_COMPILER << ")" << endl
 #else
         << "disabled" << endl
-#endif
-        << "   Disort support:       "
-#ifdef ENABLE_DISORT
-        << "enabled" << endl
-#else
-        << "disabled" << endl
 #endif
         << "   Refice support:       "
 #ifdef ENABLE_REFICE
