@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2008 Stefan Buehler <sbuehler@ltu.se>
+/* Copyright (C) 2000-2012 Stefan Buehler <sbuehler@ltu.se>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -71,6 +71,7 @@
 #include "methods.h"
 #include "workspace_ng.h"
 #include "agenda_record.h"
+#include "global_data.h"
 
 /* Adds commas and indentation to parameter lists. */
 void align(ofstream& ofs, bool& is_first_parameter, const String& indent)
@@ -272,7 +273,7 @@ void write_method_header_documentation (ofstream& ofs, const MdRecord& mdd)
 void write_method_header( ofstream& ofs,
                           const MdRecord& mdd )
 {
-  extern const ArrayOfString wsv_group_names;
+  using global_data::wsv_group_names;
   const Array<WsvRecord>& wsv_data = Workspace::wsv_data;
 
 //   // Work out the full name to use:
@@ -332,7 +333,7 @@ void write_method_header( ofstream& ofs,
   // vector as generic output, this does not mean that it is
   // the same vector!
 
-  if (mdd.Supergeneric ())
+  if (mdd.Supergeneric() && mdd.UsesTemplates())
     {
       ofs << "template <typename T>" << endl;
     }
@@ -603,8 +604,8 @@ int main()
   try
     {
       // Make the global data visible:
-      extern Array<MdRecord> md_data_raw;
-      extern Array<MdRecord> md_data;
+      using global_data::md_data_raw;
+      using global_data::md_data;
 
       // Initialize the wsv group name array:
       define_wsv_group_names();
@@ -651,6 +652,8 @@ int main()
           << "#include \"m_general.h\"\n"
           << "#include \"parser.h\"\n"
           << "#include \"workspace_ng.h\"\n"
+          << "#include \"cia.h\"\n"
+          << "#include \"linemixingrecord.h\"\n"
           << "\n";
 
       ofs << "// This is only used for a consistency check. You can get the\n"
@@ -658,22 +661,12 @@ int main()
           << "#define N_MD " << n_md << "\n\n";
 
      
-      // We don't really need these handles, do we?
-
-//       ofs << "enum MdHandle{\n";
-//       for (Index i=0; i<n_md-1; ++i)
-//      {
-//        ofs << "  " << md_data[i].Name() << "_,\n";
-//      }
-//       ofs << "  " << md_data[n_md-1].Name() << "_\n";
-//       ofs << "};\n\n";
-
       // Add all the method function declarations
       ofs << "// Method function declarations:\n\n";
       for (Index i=0; i<n_md; ++i)
         {
           const MdRecord& mdd = md_data[i];
-          if ( !mdd.SuppressHeader() )
+          if ( !mdd.UsesTemplates() )
             {
               write_method_header_documentation( ofs, mdd );
               write_method_header( ofs, mdd );
@@ -685,7 +678,7 @@ int main()
       for (Index i=0; i<md_data_raw.nelem (); ++i)
         {
           const MdRecord& mdd = md_data_raw[i];
-          if ( mdd.Supergeneric() )
+          if ( mdd.Supergeneric() && mdd.UsesTemplates() )
             {
               write_method_header_documentation( ofs, mdd );
               write_method_header( ofs, mdd );
@@ -718,7 +711,7 @@ int main()
       Workspace::define_wsv_map ();
       define_agenda_data ();
 
-      extern const Array<AgRecord> agenda_data;
+      using global_data::agenda_data;
       for (Index i = 0; i < agenda_data.nelem (); i++)
         {
           write_agenda_wrapper_header (ofs, agenda_data[i]);
