@@ -1,4 +1,4 @@
-/* Copyright (C) 2002-2008 Stefan Buehler <sbuehler@ltu.se>
+/* Copyright (C) 2002-2012 Stefan Buehler <sbuehler@ltu.se>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -25,6 +25,9 @@
 #include "matpackIII.h"
 #include "exceptions.h"
 
+using std::runtime_error;
+
+
 // Functions for ConstTensor3View:
 // ------------------------------
 
@@ -215,7 +218,7 @@ ConstTensor3View::ConstTensor3View(Numeric *data,
 /** Output operator. This demonstrates how iterators can be used to
     traverse the tensor. We use the standard output operator for
     Matrix to print each page in turn. */
-ostream& operator<<(ostream& os, const ConstTensor3View& v)
+std::ostream& operator<<(std::ostream& os, const ConstTensor3View& v)
 {
   // Page iterators:
   ConstIterator3D ip=v.begin();
@@ -438,7 +441,7 @@ Numeric *Tensor3View::get_c_array()
   if (mpr.mstart != 0 || mpr.mstride != mrr.mextent * mcr.mextent
       || mrr.mstart != 0 || mrr.mstride != mcr.mextent
       || mcr.mstart != 0 || mcr.mstride != 1)
-    throw runtime_error("A Tensor3View can only be converted to a plain C-array if mpr.mstart == 0 and mpr.mstride == mrr.extent*mcr.extent and mrr.mstart == 0 and mrr.mstride == mcr.extent and mcr.mstart == 0 and mcr.mstride == 1");
+      throw std::runtime_error("A Tensor3View can only be converted to a plain C-array if it's pointing to a continuous block of data");
 
   return mdata;
 }
@@ -454,7 +457,7 @@ const Numeric *Tensor3View::get_c_array() const
   if (mpr.mstart != 0 || mpr.mstride != mrr.mextent * mcr.mextent
       || mrr.mstart != 0 || mrr.mstride != mcr.mextent
       || mcr.mstart != 0 || mcr.mstride != 1)
-    throw runtime_error("A Tensor3View can only be converted to a plain C-array if mpr.mstart == 0 and mpr.mstride == mrr.extent*mcr.extent and mrr.mstart == 0 and mrr.mstride == mcr.extent and mcr.mstart == 0 and mcr.mstride == 1");
+      throw std::runtime_error("A Tensor3View can only be converted to a plain C-array if it's pointing to a continuous block of data");
 
   return mdata;
 }
@@ -839,13 +842,9 @@ Tensor3::Tensor3(const Tensor3& m) :
   \author Stefan Buehler
   \date   2002-12-19
 */
-Tensor3& Tensor3::operator=(const Tensor3& m)
+Tensor3& Tensor3::operator=(Tensor3 x)
 {
-  //  cout << "Tensor3 copy: m = " << m.nrows() << " " << m.ncols() << "\n";
-  //  cout << "             n = " << nrows() << " " << ncols() << "\n";
-
-  resize( m.mpr.mextent, m.mrr.mextent, m.mcr.mextent ); 
-  copy( m.begin(), m.end(), begin() );
+  swap(*this, x);
   return *this;
 }
 
@@ -887,6 +886,17 @@ void Tensor3::resize(Index p, Index r, Index c)
     }
 }
 
+
+/** Swaps two objects. */
+void swap(Tensor3& t1, Tensor3& t2)
+{
+  std::swap(t1.mpr, t2.mpr);
+  std::swap(t1.mrr, t2.mrr);
+  std::swap(t1.mcr, t2.mcr);
+  std::swap(t1.mdata, t2.mdata);
+}
+
+
 /** Destructor for Tensor3. This is important, since Tensor3 uses new to
     allocate storage. */
 Tensor3::~Tensor3()
@@ -1004,3 +1014,30 @@ Numeric debug_tensor3view_get_elem (Tensor3View& tv,
 #endif
 ////////////////////////////////
 
+//! mult Tensor3
+/*!
+    Pointwise multiplication of a vector element and matrix.
+
+    mult(Tensor3& A, const ConstVectorView B, const ConstMatrixView C) for multiplying vector pointwise with matrix.
+        Useful, e.g., for frequency gridded absorption
+        vector multiplied by normalized Stokes extinction
+        matrix to get extinction matrix as a function of frequency.
+
+    \param   A   Out: Tensor3 with N pages, M rows and L columns
+    \param   B   In: A Vector of length N.
+    \param   C   In: A Matrix of size M x L.
+
+    \author Richard Larsson
+    \date   2012-07-17
+*/
+void mult(Tensor3View A,
+          const ConstVectorView B, const ConstMatrixView C){
+    assert(A.npages() == B.nelem());
+    assert(A.nrows()  == C.nrows());
+    assert(A.ncols()  == C.ncols());
+
+    for(Index ii = 0; ii < B.nelem(); ii++){
+        A(ii,joker,joker)  = C;
+        A(ii,joker,joker) *= B[ii];
+    }
+}
\ No newline at end of file
