@@ -1,4 +1,4 @@
-/* Copyright (C) 2002-2008
+/* Copyright (C) 2002-2012
    Stefan Buehler <sbuehler@ltu.se>
    Wolfram-Andre Haas <wolhaas@hermes.fho-emden.de>
 
@@ -44,10 +44,6 @@ public:
   /** Default constructor. */
   Iterator5D() : msv(), mstride(0) { /* Nothing to do here. */ }
 
-  /** Copy constructor. */
-  Iterator5D(const Iterator5D& o) : msv(o.msv), mstride(o.mstride)
-      { /* Nothing to do here. */ }
-
   /** Explicit constructor. */
   Iterator5D(const Tensor4View& x, Index stride) : msv(x), mstride(stride)
       { /* Nothing to do here. */ }
@@ -95,10 +91,6 @@ public:
   /** Default constructor. */
   ConstIterator5D() : msv(), mstride(0) { /* Nothing to do here. */ }
 
-  /** Copy constructor. */
-  ConstIterator5D(const ConstIterator5D& o) : msv(o.msv), mstride(o.mstride)
-      { /* Nothing to do here. */ }
-
   /** Explicit constructor. */
   ConstIterator5D(const ConstTensor4View& x, Index stride)
     : msv(x), mstride(stride)
@@ -217,6 +209,12 @@ public:
       assert( r < mrr.mextent );
       assert( c < mcr.mextent );
 
+      return get(s, b, p, r, c);
+    }
+
+  /** Get element implementation without assertions. */
+  Numeric get(Index s, Index b, Index p, Index r, Index c) const
+    {
       return *( mdata +
                 msr.mstart + s * msr.mstride +
                 mbr.mstart + b * mbr.mstride +
@@ -320,6 +318,10 @@ public:
   Numeric operator()(Index s, Index b, Index p, Index r, Index c) const
     { return ConstTensor5View::operator()(s,b,p,r,c); }
 
+  /** Get element implementation without assertions. */
+  Numeric get(Index s, Index b, Index p, Index r, Index c) const
+    { return ConstTensor5View::get(s,b,p,r,c); }
+
   // Non-const index operators:
 
   Tensor5View operator()( const Range& s, const Range& b, const Range& p, const Range& r, const Range& c );
@@ -372,6 +374,12 @@ public:
       assert( r < mrr.mextent );
       assert( c < mcr.mextent );
 
+      return get(s, b, p, r, c);
+    }
+
+  /** Get element implementation without assertions. */
+  Numeric& get(Index s, Index b, Index p, Index r, Index c)
+    {
       return *( mdata +
                 msr.mstart + s * msr.mstride +
                 mbr.mstart + b * mbr.mstride +
@@ -450,12 +458,15 @@ public:
   Tensor5(const Tensor5& v);
 
   // Assignment operators:
-  Tensor5& operator=(const Tensor5& x);
+  Tensor5& operator=(Tensor5 x);
   Tensor5& operator=(Numeric x);
 
   // Resize function:
   void resize(Index s, Index b, Index p, Index r, Index c);
 
+  // Swap function:
+  friend void swap(Tensor5& t1, Tensor5& t2);
+
   // Destructor:
   virtual ~Tensor5();
 };
@@ -480,7 +491,7 @@ Numeric max(const ConstTensor5View& x);
 
 Numeric min(const ConstTensor5View& x);
 
-ostream& operator<<(ostream& os, const ConstTensor5View& v);
+std::ostream& operator<<(std::ostream& os, const ConstTensor5View& v);
 
 ////////////////////////////////
 // Helper function for debugging
