@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2008 Stefan Buehler <sbuehler@ltu.se>
+/* Copyright (C) 2001-2012 Stefan Buehler <sbuehler@ltu.se>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -41,10 +41,6 @@ public:
   /** Default constructor. */
   Iterator6D() : msv(), mstride(0) { /* Nothing to do here. */ }
 
-  /** Copy constructor. */
-  Iterator6D(const Iterator6D& o) : msv(o.msv), mstride(o.mstride)
-      { /* Nothing to do here. */ }
-
   /** Explicit constructor. */
   Iterator6D(const Tensor5View& x, Index stride) : msv(x), mstride(stride)
       { /* Nothing to do here. */ }
@@ -97,10 +93,6 @@ public:
   /** Default constructor. */
   ConstIterator6D() : msv(), mstride(0) { /* Nothing to do here. */ }
 
-  /** Copy constructor. */
-  ConstIterator6D(const ConstIterator6D& o) : msv(o.msv), mstride(o.mstride)
-      { /* Nothing to do here. */ }
-
   /** Explicit constructor. */
   ConstIterator6D(const ConstTensor5View& x, Index stride) : msv(x), mstride(stride)
       { /* Nothing to do here. */ }
@@ -387,6 +379,13 @@ public:
         CHECK(p);
         CHECK(r);
         CHECK(c);
+        return get(v, s, b, p, r, c);
+      }
+
+  /** Get element implementation without assertions. */
+  Numeric get( Index        v, Index        s, Index        b,
+               Index        p, Index        r, Index        c) const
+      {
         return                *(mdata +
                                 OFFSET(v) + OFFSET(s) + OFFSET(b) +
                                 OFFSET(p) + OFFSET(r) + OFFSET(c)    );
@@ -659,6 +658,10 @@ public:
                        Index        p, Index        r, Index        c) const
       { return ConstTensor6View::operator()(v,s,b,p,r,c);  }
 
+  /** Get element implementation without assertions. */
+  Numeric get( Index        v, Index        s, Index        b,
+               Index        p, Index        r, Index        c) const
+      { return ConstTensor6View::get(v,s,b,p,r,c);  }
 
   // Non-const index operators:
 
@@ -873,11 +876,21 @@ public:
         CHECK(p);
         CHECK(r);
         CHECK(c);
+        return get(v, s, b, p, r, c);
+      }
+
+  /** Get element implementation without assertions. */
+  Numeric& get( Index        v, Index        s, Index        b,
+                Index        p, Index        r, Index        c)
+    {
         return                *(mdata +
                                 OFFSET(v) + OFFSET(s) + OFFSET(b) +
                                 OFFSET(p) + OFFSET(r) + OFFSET(c)    );
-      }
+    }
 
+  // Conversion to a plain C-array
+  const Numeric *get_c_array() const;
+  Numeric *get_c_array();
 
   // Functions returning const iterators:
   ConstIterator6D begin() const;
@@ -947,13 +960,16 @@ public:
   Tensor6(const Tensor6& v);
 
   // Assignment operators:
-  Tensor6& operator=(const Tensor6& x);
+  Tensor6& operator=(Tensor6 x);
   Tensor6& operator=(Numeric x);
 
   // Resize function:
   void resize(Index        v, Index        s, Index        b,
               Index        p, Index        r, Index        c);
 
+  // Swap function:
+  friend void swap(Tensor6& t1, Tensor6& t2);
+
   // Destructor:
   virtual ~Tensor6();
 };
@@ -978,7 +994,7 @@ Numeric max(const ConstTensor6View& x);
 
 Numeric min(const ConstTensor6View& x);
 
-ostream& operator<<(ostream& os, const ConstTensor6View& v);
+std::ostream& operator<<(std::ostream& os, const ConstTensor6View& v);
 
 ////////////////////////////////
 // Helper function for debugging
