@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2008 Stefan Buehler <sbuehler@ltu.se>
+/* Copyright (C) 2000-2012 Stefan Buehler <sbuehler@ltu.se>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -91,20 +91,25 @@ public:
   int get_priority () const { return priority; }
   const Verbosity& get_verbosity () const { return verbosity; }
   
+  //! Does the current message have sufficient priority for output?
+  /*!
+   \return true if priority is sufficient, otherwise false. */
+  bool sufficient_priority() const
+  {
+    return (sufficient_priority_agenda()
+            && (sufficient_priority_screen() || sufficient_priority_file()));
+  }
+
   //! Does the current message have sufficient priority for agenda?
-  /*! 
-   \param priority Priority of current message.
-   
+  /*!
    \return true if priority is sufficient, otherwise false. */
   bool sufficient_priority_agenda() const
   {
-    return verbosity.get_agenda_verbosity() >= priority;
+    return (in_main_agenda() || verbosity.get_agenda_verbosity() >= priority);
   }
   
   //! Does the current message have sufficient priority for screen?
-  /*! 
-   \param priority Priority of current message.
-   
+  /*!
    \return true if priority is sufficient, otherwise false. */
   bool sufficient_priority_screen() const
   {
@@ -112,15 +117,16 @@ public:
   }
   
   //! Does the current message have sufficient priority for file?
-  /*! 
-   \param priority Priority of current message.
-   
+  /*!
    \return true if priority is sufficient, otherwise false. */
   bool sufficient_priority_file() const
   {
     return verbosity.get_file_verbosity() >= priority;
   }
   
+  //! Are we in the main agenda?
+  /*!
+   \return true if in main agenda, otherwise false. */
   bool in_main_agenda() const
   {
     return verbosity.is_main_agenda();
@@ -164,7 +170,7 @@ ArtsOut& operator<<(ArtsOut& aos, const T& t)
   // output must be fulfilled in addition to the condition for
   // screen or file. 
   
-  if (aos.in_main_agenda() || aos.sufficient_priority_agenda())
+  if (aos.sufficient_priority_agenda())
   {
     // We are marking the actual output operations as omp
     // critical, to somewhat reduce the mess when several threads
@@ -176,18 +182,18 @@ ArtsOut& operator<<(ArtsOut& aos, const T& t)
     
     if (aos.sufficient_priority_screen())
     {
-#pragma omp critical
+#pragma omp critical (ArtsOut_screen)
       {
         if (aos.get_priority() == 0)
-          cerr << t;
+            cerr << t << flush;
         else
-          cout << t;
+            cout << t << flush;
       }
     }
     
     if (aos.sufficient_priority_file())
     {
-#pragma omp critical
+#pragma omp critical (ArtsOut_file)
       {
         //    if (report_file)              // Check if report file is good
         report_file << t << flush;
@@ -202,15 +208,15 @@ ArtsOut& operator<<(ArtsOut& aos, const T& t)
   return aos;
 }
 
-#define CREATE_OUT0 ArtsOut0 out0(verbosity);
-#define CREATE_OUT1 ArtsOut1 out1(verbosity);
-#define CREATE_OUT2 ArtsOut2 out2(verbosity);
-#define CREATE_OUT3 ArtsOut3 out3(verbosity);
+#define CREATE_OUT0 ArtsOut0 out0(verbosity)
+#define CREATE_OUT1 ArtsOut1 out1(verbosity)
+#define CREATE_OUT2 ArtsOut2 out2(verbosity)
+#define CREATE_OUT3 ArtsOut3 out3(verbosity)
 
 #define CREATE_OUTS \
 ArtsOut0 out0(verbosity); \
 ArtsOut1 out1(verbosity); \
 ArtsOut2 out2(verbosity); \
-ArtsOut3 out3(verbosity);
+ArtsOut3 out3(verbosity)
 
 #endif // messages_h
