@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2008 Stefan Buehler <sbuehler@ltu.se>
+/* Copyright (C) 2001-2012 Stefan Buehler <sbuehler@ltu.se>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -30,6 +30,7 @@
 #include <cassert>
 #include <climits>
 #include <string>
+#include <algorithm>
 #include "matpack.h"
 #include "array.h"
 
@@ -41,6 +42,7 @@
 #include "sstream.h"
 #endif
 
+
 /**
    The implementation for String, the ARTS string class. 
 
@@ -58,13 +60,13 @@
    not from string directly.
 */
 template<class charT>
-class my_basic_string : public basic_string<charT>
+class my_basic_string : public std::basic_string<charT>
 {
 public:
   // Constructors:
   my_basic_string();
   explicit my_basic_string(Index n, char c=' ');
-  my_basic_string(const basic_string<charT>& A,
+  my_basic_string(const std::basic_string<charT>& A,
                   Index pos=0,
                   Index numpos=my_basic_string<charT>::npos);
   my_basic_string(const char A[]);
@@ -78,22 +80,29 @@ public:
                      const my_basic_string<charT>& insstr);
 
   // Split string
-  void split (Array< my_basic_string<charT> > &aos,
+  void split(Array< my_basic_string<charT> > &aos,
               const my_basic_string<charT> &delim) const;
   
+  /** Convert to upper case */
+  void toupper() { std::transform ( this->begin(),  this->end(),
+                                    this->begin(), ::toupper); }
+
+  /** Convert to lower case */
+  void tolower() { std::transform ( this->begin(),  this->end(),
+                                    this->begin(), ::tolower); }
+  
+  /** Trim leading and trailing whitespace */
+  void trim();
+
   // Number of elements:
   Index nelem() const;
 
-  // Find functions:
-//   Index find(char c);
-//   Index find(const my_basic_string<charT>&  c);
-
   // Index operators:
   char operator[](Index n) const;
   char& operator[](Index n);
 
   /** Define npos: */
-  static const Index npos = static_cast<Index>(basic_string<charT>::npos);
+  static const Index npos = static_cast<Index>(std::basic_string<charT>::npos);
 
   typedef Index size_type;
 };
@@ -106,9 +115,10 @@ public:
 
 /** Default constructor. */
 template<class charT>
-inline my_basic_string<charT>::my_basic_string() : basic_string<charT>()  
+inline my_basic_string<charT>::my_basic_string() : std::basic_string<charT>()
 { /* Nothing to do here. */ }
 
+
 /** Constructor setting size. You may give as a second argument a
     character with which to fill the new string. Per default this is
     zero. 
@@ -118,13 +128,9 @@ inline my_basic_string<charT>::my_basic_string() : basic_string<charT>()
 */
 template<class charT>
 inline my_basic_string<charT>::my_basic_string(Index n, char c) :
-  basic_string<charT>(n,c) 
+  std::basic_string<charT>(n,c)
 { /* Nothing to do here. */ }
 
-/** Construnctor from another my_basic_string. */
-// template<class charT>
-// inline my_basic_string<charT>::my_basic_string(const my_basic_string& A) : basic_string<charT>(A) 
-// { /* Nothing to do here. */ };
 
 /** Construnctor from a basic_string. This is important for handling
     of expressions like this to work correctly:
@@ -139,23 +145,25 @@ inline my_basic_string<charT>::my_basic_string(Index n, char c) :
     \param numpos How many characters to copy
 */
 template<class charT>
-inline my_basic_string<charT>::my_basic_string(const basic_string<charT>& A,
+inline my_basic_string<charT>::my_basic_string(const std::basic_string<charT>& A,
                                                Index pos,
                                                Index numpos)
 { 
   // Range checks:
   assert(0<=pos);               // Start index must be 0 or greater 0.
 
+  if (!A.size()) return;
+
 //   cout << "A = " << A << "\n";
 //   cout << "pos = " << pos << "\n";
 //   cout << "size = " << A.size() << "\n";
 
-  assert(static_cast<typename basic_string<charT>::size_type>(pos)<A.size());      
+  assert(static_cast<typename std::basic_string<charT>::size_type>(pos)<A.size());
   // At most the last element of the original string.
 
   assert( numpos==my_basic_string<charT>::npos ||
           ( (numpos >= 0) &&
-            (static_cast<typename basic_string<charT>::size_type>(numpos)<=(A.size()-pos))
+            (static_cast<typename std::basic_string<charT>::size_type>(numpos)<=(A.size()-pos))
             )
           );  // Number of characters to copy must be at the most the
               // number left. -1 means all remaining characters. 
@@ -167,13 +175,13 @@ inline my_basic_string<charT>::my_basic_string(const basic_string<charT>& A,
   // case npos -1 (=my_basic_string<charT>::npos) is also handled
   // correctly.)
 
-  basic_string<charT>::operator=(basic_string<charT>(A,pos,numpos));
+  std::basic_string<charT>::operator=(std::basic_string<charT>(A,pos,numpos));
 
 }
 
 /** Constructor from a C-style char array. */
 template<class charT>
-inline my_basic_string<charT>::my_basic_string(const char A[]) : basic_string<charT>(A) 
+inline my_basic_string<charT>::my_basic_string(const char A[]) : std::basic_string<charT>(A)
 { /* Nothing to do here. */ }
 
 
@@ -186,10 +194,11 @@ inline my_basic_string<charT>::my_basic_string(const char A[]) : basic_string<ch
 template<class charT>
 inline my_basic_string<charT>& my_basic_string<charT>::operator=(const my_basic_string<charT>& A)
 {
-  basic_string<charT>::operator=(A);
+  std::basic_string<charT>::operator=(A);
   return *this;
 }
 
+
 /** Insert string before all occurrences of the substring.
  
  \param[in] searchstr  String to search for.
@@ -203,10 +212,10 @@ inline void my_basic_string<charT>::insert_substr(const my_basic_string<charT>&
   size_t insstr_size = insstr.size();
   size_t start_pos = 0;
 
-  while (start_pos != string::npos)
+  while (start_pos != std::string::npos)
     {
       start_pos = this->find (searchstr, start_pos);
-      if (start_pos && start_pos != string::npos)
+      if (start_pos && start_pos != std::string::npos)
         {
           this->insert (start_pos, insstr);
           start_pos += searchstr_size + insstr_size;
@@ -214,6 +223,7 @@ inline void my_basic_string<charT>::insert_substr(const my_basic_string<charT>&
     }
 }
 
+
 /** Split string into substrings.
  
  \param[out] aos    ArrayOfString containing the returned substrings.
@@ -230,13 +240,39 @@ inline void my_basic_string<charT>::split (Array< my_basic_string<charT> > &aos,
   while (oldpos < (size_t)this->nelem() &&
          (pos = this->find(delim, oldpos)) != (size_t)my_basic_string<charT>::npos)
   {
-    if (pos) aos.push_back(this->substr(oldpos, pos-oldpos));
+    if (pos && pos-oldpos) aos.push_back(this->substr(oldpos, pos-oldpos));
     oldpos = pos+delim.nelem();
   }
   
   if (oldpos < (size_t)this->nelem()) aos.push_back(this->substr(oldpos));
 }
 
+
+/** Trim leading and trailing whitespace */
+template<class charT>
+void my_basic_string<charT>::trim()
+{
+    // Create ref to self for readability
+    my_basic_string& this_string = *this;
+
+    // Remove leading whitespace
+    while (0 != this_string.nelem()
+           && (' ' == this_string[0]
+               || '\t' == this_string[0]
+               || '\n' == this_string[0]
+               || '\r' == this_string[0]))
+        this_string.erase(0,1);
+
+    // Remove trailing whitespace
+    while (0 != this_string.nelem()
+           && (' ' == this_string[this_string.nelem()-1]
+               || '\t' == this_string[this_string.nelem()-1]
+               || '\n' == this_string[this_string.nelem()-1]
+               || '\r' == this_string[this_string.nelem()-1]))
+        this_string.erase(this_string.nelem()-1);
+}
+
+
 /** Number of elements. */
 template<class charT>
 inline Index my_basic_string<charT>::nelem() const
@@ -246,6 +282,7 @@ inline Index my_basic_string<charT>::nelem() const
   return static_cast<long>(s);
 }
 
+
 /**
   Constant index operator. We redifine this here so that we can have
   range checking by assert.
@@ -257,9 +294,10 @@ inline char my_basic_string<charT>::operator[](Index n) const
 {
   assert(0<=n);
   assert(n<nelem());
-  return basic_string<charT>::operator[](n);
+  return std::basic_string<charT>::operator[](n);
 }
 
+
 /**
   Non-constant index operator. We redifine this here so that we can
   have range checking by assert.
@@ -271,73 +309,50 @@ inline char& my_basic_string<charT>::operator[](Index n)
 {
   assert(0<=n);
   assert(n<nelem());
-  return basic_string<charT>::operator[](n);
+  return std::basic_string<charT>::operator[](n);
 }
 
-// Non-member functions:
-
-// /** Output operator. */
-// inline ostream& operator<<(ostream& os, const my_basic_string& v)
-// {
-//   my_basic_string<base>::const_iterator         i = v.begin();
-//   const my_basic_string<base>::const_iterator end = v.end();
-
-//   if ( i!=end )
-//     {
-//       os << *i;
-//       ++i;
-//     }
-
-//   for ( ; i!=end; ++i )
-//     {
-//       os << "\n" << setw(3) << *i;
-//     }
-
-
-//   // Just use the operator of string.
-//   operator<<(os,v);
-//   return os;
-// }
-
 
 /** The String type for ARTS. Implementation see documentation of
     class my_basic_string. */
 typedef my_basic_string<char> String;
 
-// Declare the existance of class Array:
-/*template<class base>
-class Array;*/
-
 /** An array of Strings. */
 typedef Array<String> ArrayOfString;
 
-//
-// We don't use this function, we use string.find() instead.
-//
-// //! Find first occurance.
-// /*!
-//   This returns the index of the first occurance of w in 
-//   string x.  
-// 
-//   A return value of -1 indicates that no matching element was found.
-// 
-//   \return   The index of the thing we looked for.
-//   \param  x The string to search.
-//   \param w  The character to look for.
-// 
-//   \author Stefan Buehler
-//   \date   2002-12-06
-// */
-// template <class base>
-// Index find_first( const my_basic_string<base>& x,
-//                   const base& w )
-// {
-//   for ( Index i=0; i<x.nelem(); ++i )
-//     if ( w == x[i] )
-//       return i;
-// 
-//   return -1;
-// }
 
+/** Extract something from the beginning of a string. This is just a small helper
+ function to safe some typing.
+
+ \retval x    What was extracted from the beginning of the line.
+ \retval line What was extracted is also cut away from line.
+ \param n     The width of the stuff to extract.
+
+ \author Stefan Buehler */
+template<class T>
+void extract(T&      x,
+             String& line,
+             Index  n)
+{
+    // Initialize output to zero! This is important, because otherwise
+    // the output variable could `remember' old values.
+    x = T(0);
+
+    // This will contain the short subString with the item to extract.
+    // Make it a String stream, for easy parsing,
+    // extracting subString of width n from line:
+    std::istringstream item( line.substr(0,n) );
+
+    //  cout << "line = '" << line << "'\n";
+    //   cout << "line.substr(0,n) = " << line.substr(0,n) << endl;
+    //   cout << "item = " << item.str() << endl;
+
+    // Shorten line by n:
+    line.erase(0,n);
+    //  cout << "line = " << line << endl;
+
+    // Convert with the aid of String stream item:
+    item >> x;
+}
 
 #endif  // mystring_h
